#include "renderer.hpp"

namespace gz::vk {
    class Renderer2D : public Renderer {
        public:
            Renderer2D(VulkanInstance& instance);
            void drawShape(const Shape& shape);
            /**
             * @brief Copies the vertices from shapes into the vertex buffer, using a staging buffer
             */
            void fillVertexBufferWithShapes();
            void fillIndexBufferWithShapes();
            void drawFrame(uint32_t imageIndex);
            void cleanup();
            /**
             * @brief Create a render pass
             * @details
             *  Attachments:
             *  - color blend:
             *      - loadOp = VK_ATTACHMENT_LOAD_OP_LOAD (not clear!)
             *      - storeOp = VK_ATTACHMENT_STORE_OP_STORE
             *      - initialLayout = VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL
             *      - finalLayout = VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL
             *      - stencil load/store = dont care
             */
            void createRenderPass();

        private:
            std::vector<Shape> shapes;
            size_t shapesVerticesCount = 0;
            size_t shapesIndicesCount = 0;
            /**
             * @brief Render everything
             * @details
             *  As of now, this does (in the same command buffer from graphicsPool)
             *  -# begin render pass 
             *      - image layout: VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL
             *      - clear image
             *  -# bind 2d pipeline, vertex and index buffer
             *  -# draw indexed: draw the shapes from shapes vector
             *  -# end render pass 
             *      - image layout: VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL
             *  -# copy image to swapChain image with same imageIndex
             *  
             */
            void recordCommandBuffer(uint32_t imageIndex, uint32_t currentFrame);
            // IMAGE
            std::vector<VkImage> images;
            std::vector<VkDeviceMemory> imageMemory;
            std::vector<VkImageView> imageViews;
            /**
             * @brief Creates the images and imageViews with the format of the VulkanInstance::swapChain images
             */
            void createImages();
            VkRenderPass renderPass;
            int renderPassID;

            Log rLog;

    };

}  // namespace gz::vk
